/*********************************************************************************************/
/*                                                                                           */
/*                                                                          File: Logger.cpp */
/*                                                                                           */
/*     Logs the messages of the processes                                                    */
/*     ==================================                                                    */
/*                                                                                           */
/*     V0.1     25-APR-2017      Te                                                          */
/*     V0.2     27-APR-2017      Te                                                          */
/*     V0.3     07-JUL-2017      Te   Added GPL license                                      */
/*                                                                                           */
/*********************************************************************************************/

/*
 *  Logger for Swissphone TPS 
 *  Copyright (C) 2017 P. Tellenbach
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include <stdio.h>
#include <time.h>
#include <stdarg.h>

static const char *logfile ;

/***********************************/
 void SetLogfile( const char *file )
/***********************************/

{
   logfile = file ;
}

/****************************************/
 void WriteLog( const char *format, ... )
/****************************************/

{
   time_t     now = time( NULL ) ;
   struct tm *pt  = localtime( &now ) ;
   va_list    args ;

   if( logfile != NULL )
   {
      FILE *log = fopen( logfile, "a" ) ;

      if( log == NULL )
      {
         fprintf( stdout, "error: can't open logfile\n" ) ;
         return ;
      }

      fprintf( log, "%02d.%02d.%d %02d:%02d ", pt->tm_mday, pt->tm_mon + 1, pt->tm_year + 1900, pt->tm_hour, pt->tm_min ) ;

      va_start( args, format ) ;
      vfprintf( log, format, args ) ;
      va_end( args ) ;
   
      fprintf( log, "\n" ) ;
      fclose( log ) ;
   }
   else
   {
      printf( "%02d.%02d.%d %02d:%02d ", pt->tm_mday, pt->tm_mon + 1, pt->tm_year + 1900, pt->tm_hour, pt->tm_min ) ;

      va_start( args, format ) ;
      vprintf( format, args ) ;
      va_end( args ) ;
   
      printf( "\n" ) ;
   }
}
