/******************************************************************************/
/*                                                                            */
/*                                                         FILE: Swissphone.h */
/*                                                                            */
/*        Definitions for the TAP protocol of Swissphone                      */
/*        ==============================================                      */
/*                                                                            */
/*        V0.01   20-MAR-2017  Te                                             */
/*                                                                            */
/******************************************************************************/

/*
 *  A class which implements the TAP protocol for Swissphone TPS 
 *  Copyright (C) 2017 P. Tellenbach
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef SWISSPHONE_H
#define SWISSPHONE_H

#include "configuration.h"

#define SE_CONNECT  206
#define SE_CONFIG   208
#define SE_QUEUE    209
#define SE_RECEIVE  210

class Swissphone
{
 private:
   char buffer[1024] ;
   int  index ;
   int  sock ;

   int  Receive( int seconds ) ;
   int  CheckResponse( int seconds, int (*match)(const char *str) ) ;
   bool StartProtocol() ;
   int  Accepted() ;

 public:
   bool Connect( const char *host, const char *port ) ;
   int  Login( const char *user, const char *password ) ;
   bool CenterReady() ;
   int  SendPage( const PAGER &pager, const char *text ) ;
   void Logout() ;
   void Disconnect() ;
} ;

#endif
